package gov.usgs.earthquake.nshmp.www;

import java.math.BigDecimal;
import java.util.List;
import java.util.logging.Logger;

import org.slf4j.LoggerFactory;

import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.model.HazardModel;
import gov.usgs.earthquake.nshmp.www.source.FeaturesService;

import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.MediaType;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.swagger.v3.core.util.Yaml;
import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.parameters.Parameter;
import io.swagger.v3.parser.OpenAPIV3Parser;
import jakarta.inject.Inject;

/**
 * Expose OpenAPI YAML file.
 *
 * @author U.S. Geological Survey
 */
@Tag(
    name = "Swagger",
    description = "Swagger OpenAPI YAML")
@Hidden
@Controller("/swagger")
public class SwaggerController {

  @Inject
  private NshmpMicronautServlet servlet;

  @Get(produces = MediaType.TEXT_EVENT_STREAM)
  public HttpResponse<String> doGet(HttpRequest<?> request) {
    try {
      var model = ServletUtil.model();
      var openApi = getOpenAPI(request, model);
      return HttpResponse.ok(Yaml.pretty(openApi));
    } catch (Exception e) {
      return ServletUtil.error(
          LoggerFactory.getLogger("Swagger"),
          e, "Swagger", request.getUri().getPath());
    }
  }

  private OpenAPI getOpenAPI(
      HttpRequest<?> request,
      HazardModel model) {
    Logger log = Logger.getAnonymousLogger();
    var openApi = new OpenAPIV3Parser().read("META-INF/swagger/nshmp-haz.yml");
    var bounds = model.bounds();
    var components = openApi.getComponents();
    var schemas = components.getSchemas();
    SwaggerUtils.siteClassSchema(schemas, List.copyOf(model.siteClasses().keySet()));
    SwaggerUtils.imtSchema(schemas, List.copyOf(model.config().hazard.imts));
    FeaturesService.featureTypeSchema(schemas);
    openApi.servers(null);

    openApi.getInfo().setTitle(model.name() + " Web Services");
    openApi.getInfo().setDescription(
        "National Seismic Hazard Model (NSHM) hazard calculations and queries for the " +
            model.name() + " hazard model.");

    return openApi;
  }

  public static OpenAPI addLocationBounds(OpenAPI openApi, Location min, Location max) {
    openApi.getPaths().values().stream()
        .flatMap(path -> path.readOperations().stream())
        .forEach(operation -> addLocationBounds(operation.getParameters(), min, max));

    return openApi;
  }

  public static List<Parameter> addLocationBounds(
      List<Parameter> parameters,
      Location min,
      Location max) {
    var latitudeDescription = String.format(" [%s, %s]", min.latitude, max.latitude);
    var longtudeDescription = String.format(" [%s, %s]", min.longitude, max.longitude);

    parameters.forEach(parameter -> {
      if (parameter.getName().equals("latitude")) {
        parameter.setDescription(parameter.getDescription() + latitudeDescription);
        parameter.getSchema().setMinimum(BigDecimal.valueOf(min.latitude));
        parameter.getSchema().setMaximum(BigDecimal.valueOf(max.latitude));
      } else if (parameter.getName().equals("longitude")) {
        parameter.setDescription(parameter.getDescription() + longtudeDescription);
        parameter.getSchema().setMinimum(BigDecimal.valueOf(min.longitude));
        parameter.getSchema().setMaximum(BigDecimal.valueOf(max.longitude));
      }
    });

    return parameters;
  }

}
